#include "Minuit2/FCNBase.h"
#include "TH1F.h"
#include "TMatrixD.h"

#include <vector>

using namespace ROOT;
using namespace Minuit2;

class Chi2FCN : public FCNBase {

    public:
        Chi2FCN(int nTemplates, int nDec);
        ~Chi2FCN();

        virtual double Up() const {return theErrorDef;}
        virtual double operator()(const std::vector<double>&) const;

        // set the observable variables 
        //
        // sgnMC are the signal templates predicted by MC: is an array 
        // of histograms, where each index contains the MC distribution 
        // of the variable for a given template 
        //
        // bkgMC are the background templates predicted by MC: is a single
        // histogram for each observable
        //
        // theData is the corresponding measured variable
        void SetObs(TH1F** sgnMC, TH1F* bkgMC, TH1F* theData);

        // set the covariance matrix
        void SetCovariance(TMatrixD* covVb, TMatrixD* covVc);

        void SetDebug() { fDebug = true; }

        void SetErrorDef(double def) {theErrorDef = def;}

    private:

        // sets the 3D T matrix: used only internally by the class
        void SetMatrix();

        // the signal MC observables: variables set through SetObs() method
        std::vector<TH1F**> hSgnMC;

        // the background MC observables: variables set through SetObs() method
        std::vector<TH1F*> hBkgMC;

        // measured variables
        std::vector<TH1F*> hData;

        // the 3D matrix
        double*** T;

        bool IsMatrixSet;

        // background array template
        double* Nbkg;

        // sgn covariance matrix
        TMatrixD* invCovVb;
        
        // background covariance matrix
        TMatrixD* invCovVc;

        // n bins in neutrino spectrum
        int fNTemp;

        // tot bins in observables
        int fNBinsObs;

        // n decays
        int fNDec;

        // measured observables
        double* Nmeas;

        bool IsDataSet;

        double theErrorDef;

        bool fDebug;

};
